/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Sun Microsystems, Inc.
 */
package org.netbeans.modules.kenai.ui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.io.Serializable;
import java.util.logging.Logger;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.SwingUtilities;
import org.netbeans.modules.kenai.api.Kenai;
import org.netbeans.modules.kenai.api.KenaiManager;
import org.netbeans.modules.kenai.ui.dashboard.DashboardImpl;
import org.netbeans.modules.kenai.ui.nodes.AddInstanceAction;
import org.netbeans.modules.kenai.ui.spi.UIUtils;
import org.openide.awt.Mnemonics;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;

/**
 * Top component which displays something.
 * @author Jan Becicka
 */
public final class KenaiTopComponent extends TopComponent {

    private static KenaiTopComponent instance;
    /** path to the icon used by the component and its open action */
    static final String ICON_PATH = "org/netbeans/modules/kenai/ui/resources/kenai-small.png"; // NOI18N
    private static final String PREFERRED_ID = "KenaiTopComponent"; // NOI18N
    private JComboBox combo;

    private JComponent dashboardComponent;

    private KenaiTopComponent() {
        initComponents();
        setName(NbBundle.getMessage(KenaiTopComponent.class, "CTL_KenaiTopComponent"));
        setToolTipText(NbBundle.getMessage(KenaiTopComponent.class, "HINT_KenaiTopComponent"));
        setIcon(ImageUtilities.loadImage(ICON_PATH, true));
        setLayout(new BorderLayout());
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        setLayout(new java.awt.BorderLayout());
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    /**
     * Gets default instance. Do not use directly: reserved for *.settings files only,
     * i.e. deserialization routines; otherwise you could get a non-deserialized instance.
     * To obtain the singleton instance, use {@link #findInstance}.
     */
    public static synchronized KenaiTopComponent getDefault() {
        if (instance == null) {
            instance = new KenaiTopComponent();
        }
        return instance;
    }

    /**
     * Obtain the KenaiTopComponent instance. Never call {@link #getDefault} directly!
     */
    public static synchronized KenaiTopComponent findInstance() {
        TopComponent win = WindowManager.getDefault().findTopComponent(PREFERRED_ID);
        if (win == null) {
            Logger.getLogger(KenaiTopComponent.class.getName()).warning(
                    "Cannot find " + PREFERRED_ID + " component. It will not be located properly in the window system."); // NOI18N
            return getDefault();
        }
        if (win instanceof KenaiTopComponent) {
            return (KenaiTopComponent) win;
        }
        Logger.getLogger(KenaiTopComponent.class.getName()).warning(
                "There seem to be multiple components with the '" + PREFERRED_ID + // NOI18N
                "' ID. That is a potential source of errors and unexpected behavior."); // NOI18N
        return getDefault();
    }

    @Override
    public int getPersistenceType() {
        return TopComponent.PERSISTENCE_ALWAYS;
    }

    @Override
    public void componentOpened() {
        removeAll();
        dashboardComponent = DashboardImpl.getInstance().getComponent();
        add(dashboardComponent, BorderLayout.CENTER);
        add(getKenaiSwitcher(), BorderLayout.NORTH);
    }

    Component getKenaiSwitcher() {
        combo = new KenaiCombo(false);
        combo.addActionListener(new ActionListener() {

            public void actionPerformed(final ActionEvent e) {
                if (combo.getSelectedItem() instanceof Kenai) {
                    DashboardImpl.getInstance().setKenai((Kenai) combo.getSelectedItem());
                } else if (combo.getSelectedItem() instanceof String) {
                    SwingUtilities.invokeLater(new Runnable() {
                        @Override
                        public void run() {
                            new AddInstanceAction().actionPerformed(e);
                            DashboardImpl.getInstance().setKenai((Kenai) combo.getSelectedItem());
                        }
                    });
                } else {
                    DashboardImpl.getInstance().setKenai(null);
                }
            }
        });

        JLabel serverLabel = new JLabel();
        final JPanel panel = new JPanel();
        java.awt.GridBagConstraints gridBagConstraints;

        serverLabel = new javax.swing.JLabel();
        JSeparator jSeparator = new javax.swing.JSeparator();

        panel.setBorder(javax.swing.BorderFactory.createEmptyBorder(3, 3, 0, 3));
        panel.setLayout(new java.awt.GridBagLayout());

        Mnemonics.setLocalizedText(serverLabel, NbBundle.getMessage(KenaiTopComponent.class, "LBL_Server"));
        serverLabel.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 1, 3, 3));
        panel.add(serverLabel, new java.awt.GridBagConstraints());

        combo.setBorder(javax.swing.BorderFactory.createEmptyBorder(1, 1, 3, 3));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        panel.add(combo, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        panel.add(jSeparator, gridBagConstraints);

        panel.setBackground(dashboardComponent.getBackground());
        combo.setOpaque(false);

        combo.addComponentListener(new ComponentAdapter() {

            @Override
            public void componentShown(ComponentEvent e) {
                panel.setVisible(true);
            }

            @Override
            public void componentHidden(ComponentEvent e) {
                panel.setVisible(false);
            }
        });

        return panel;
    }

    public void setSelectedKenai(Kenai kenai) {
        if (combo!=null) {
            combo.setSelectedItem(kenai);
        }
    }

    @Override
    public void componentClosed() {
        DashboardImpl.getInstance().close();
        dashboardComponent = null;
        removeAll();
    }

    @Override
    public void componentActivated() {
        super.componentActivated();
        Kenai pref = Utilities.getPreferredKenai();
        if (pref!=null && pref.getPasswordAuthentication() != null
                || DashboardImpl.getInstance().getOpenProjects().length > 0) {
            UIUtils.logKenaiUsage("DASHBOARD"); // NOI18N
        }
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean requestFocusInWindow() {
        // Needed to transfer focus to dashboard e.g. when switched to it from Window menu.
        boolean b = super.requestFocusInWindow();
        if (dashboardComponent != null) {
            b = dashboardComponent.requestFocusInWindow();
        }
        return b;
    }

    @SuppressWarnings("deprecation")
    @Override
    public void requestFocus() {
        // Needed to transfer focus to dashboard e.g. if it is the active TopComponent after restart.
        super.requestFocus();
        if (dashboardComponent != null) {
            dashboardComponent.requestFocus();
        }
    }

    /** replaces this in object stream */
    @Override
    public Object writeReplace() {
        return new ResolvableHelper();
    }

    @Override
    protected String preferredID() {
        return PREFERRED_ID;
    }

    final static class ResolvableHelper implements Serializable {

        private static final long serialVersionUID = 1L;

        public Object readResolve() {
            return KenaiTopComponent.getDefault();
        }
    }
}
